#Load the c code through dynamic loading, need to run R CMD SHLIB first to create .so from .c
dyn.load("algorithm1.so")


# Wrapper function for algorithm1 that uses the c code
# Q - symmetric spd matrix, p x p
# lambda - tuning parameter, >=0
# v_init - initial starting point
# eps - convergence criterion
# max_iter - maximal number of iterations
algorithm1_c<-function(Q,lambda, v_init=NULL, max_iter=1000, eps=1e-4){
    p=ncol(Q)
    # check whether the initial starting point is provided
    if (is.null(v_init)){
        library(irlba)
        # initialize v as the leading eigenvector of Q
        v_init=irlba(Q,nu=1,nv=0)$u
        # normalize v to have L2 norm equal to one
        v_init=v_init/(sqrt(as.numeric(crossprod(v_init))))
    }
    v=v_init
    Qv=Q%*%v
    iter=0
    
    out=.C("algorithm1",as.double(Q),as.double(lambda),as.double(v),as.double(Qv),as.integer(p),as.double(eps),as.integer(max_iter),as.integer(iter))
    
    # check whether the algorithm stopped because of the convergence or because the maximal number of iterations was reached
    if (out[[8]]==max_iter){warning("Failed to converge, try increasing the number of iterations")}
    
    fobj=crossprod(out[[4]],out[[3]])-lambda*sum(abs(out[[3]]))
    #check whether the objective is negative
    if (fobj<0){
        return(list(v=rep(0,p),fobj=0))
    }
    
    # return the local solution and the corresponding value of the objective function
    return(list(v=out[[3]],fobj=fobj))
}