# Load the colon canser dataset 
library(plsgenomics)
data(Colon)
ytrain=Colon$Y
xtrain=Colon$X
# Construct matrix W
p=ncol(xtrain)
G=max(ytrain)
W=((sum(ytrain==1)-1)*cov(xtrain[ytrain==1,])+(sum(ytrain==2)-1)*cov(xtrain[ytrain==2,]))/length(ytrain)
# Scale so that diag(W)=I
xtrains=scale(xtrain,center=F,scale=sqrt(diag(W)))
# Construct matrix B
B=(sum(ytrain==1)*(colMeans(xtrains[ytrain==1,])-colMeans(xtrains))%*%t(colMeans(xtrains[ytrain==1,])-colMeans(xtrains))+sum(ytrain==2)*(colMeans(xtrains[ytrain==2,])-colMeans(xtrains))%*%t(colMeans(xtrains[ytrain==2,])-colMeans(xtrains)))/length(ytrain)

# Apply Algorithm~1 for the range of tuning parameters
source("Algorithm1.R")

lambda_max=max(sqrt(rowSums(B^2)))
nl=100
lambda_intervalC=seq(0,lambda_max,by=(lambda_max)/(nl-1))
featuresC=rep(0,nl)
l1_normC=rep(0,nl)
objectiveC=rep(0,nl)

#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features
for (i in 1:length(lambda_intervalC)){
    cat(i)
    lambda=lambda_intervalC[i]
    v=algorithm1(B,lambda)$v
    featuresC[i]=sum(v!=0)
    l1_normC[i]=sum(abs(v))
    objectiveC[i]=crossprod(v,B%*%v)-lambda*sum(abs(v))
    if (featuresC[i]==0) break
}
#Make a plot of number of selected variables versus tuning parameter
pdf(file="ColonCancerLDA.pdf",width=4,height=4)
library(ggplot2)
scenarioC=data.frame(lambda=lambda_intervalC,features=featuresC)
ggplot(scenarioC,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("LDA on Colon Cancer Dataset")
dev.off()


# Drop prediction
############
source("CalculateMlambda.R")
Q=B
# m_lambda
m_lambda=findMlambda(Q,lambda_intervalC) #predicts 38

#Make a plot of number of selected variables versus tuning parameter
pdf(file="ColonCancerLDAmlambda.pdf",width=4,height=4)
library(ggplot2)
scenarioC=data.frame(lambda=lambda_intervalC,features=featuresC,Mlambda=m_lambda)
ggplot(scenarioC,aes(x=lambda,y=features))+geom_point(size=2)+geom_line(data=scenarioC,aes(x=lambda,y=Mlambda),linetype="dashed",size=2,colour="blue")+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("LDA on Colon Cancer Dataset")
dev.off()

###################################
## Consider constrained version
##########################
#Find eigenvector and eigenvalue of B
library(irlba)
tmp=irlba(B,nu=1,nv=0)
l=tmp$u
gamma=tmp$d
tau_intervalC=seq(1,sum(abs(l)),by=(sum(abs(l)))/50)
featuresCtau=rep(0,length(tau_intervalC))
source("Algorithm1.R")

#Solve using Algorithm 2 for the range of tau_values, keep track of the number of selected features
for (i in 1:length(tau_intervalC)){
    cat(i)
    tau=tau_intervalC[i]
    v=algorithm2(B,tau)
    featuresCtau[i]=sum(v!=0)
}

#Make a plot of number of selected variables versus tuning parameter for Scenario 2
library(ggplot2)
pdf(file="ColonCancerLDAconstrained.pdf",width=4,height=4)
scenarioCtau=data.frame(lambda=-tau_intervalC,features=featuresCtau)
ggplot(scenarioCtau,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(-tau))+ggtitle("LDA on Colon Cancer Dataset")
dev.off()

