# Load the colon canser dataset 
library(plsgenomics)
data(Colon)
ytrain=Colon$Y
xtrain=Colon$X
p=ncol(xtrain)

xtrains=scale(xtrain)
Q=crossprod(xtrains)/(nrow(xtrains)-1)

# Apply Algorithm~1 for the range of tuning parameters
source("Algorithm1.R")
lambda_max=max(sqrt(rowSums(Q^2)))
nl=80
lambda_intervalCPCA=seq(0,lambda_max,by=(lambda_max)/(nl-1))
featuresCPCA=rep(0,nl)
l1_normCPCA=rep(0,nl)
objectiveCPCA=rep(0,nl)
#Solve using Algorithm 1 for the range of lambda_values, keep track of the number of selected features
for (i in 1:length(lambda_intervalCPCA)){
    cat(i)
    lambda=lambda_intervalCPCA[i]
    v=algorithm1(Q,lambda)
    featuresCPCA[i]=sum(v!=0)
    l1_normCPCA[i]=sum(abs(v))
    objectiveCPCA[i]=crossprod(v,Q%*%v)-lambda*sum(abs(v))
    if (featuresCPCA[i]==0) break
}

pdf(file="ColoncancerPCA.pdf",width=4,height=4)
library(ggplot2)
scenarioCPCA=data.frame(lambda=lambda_intervalCPCA,features=featuresCPCA)
ggplot(scenarioCPCA,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("PCA on Colon Cancer Dataset")
dev.off()

# Drop prediction
############
source("CalculateMlambda.R")
# m_lambda
m_lambda=findMlambda(Q,lambda_intervalCPCA) #predicts 

#Make a plot of number of selected variables versus tuning parameter
pdf(file="ColonCancerPCAmlambda.pdf",width=4,height=4)
library(ggplot2)
scenarioCPCA=data.frame(lambda=lambda_intervalCPCA,features=featuresCPCA,Mlambda=m_lambda)
ggplot(scenarioCPCA,aes(x=lambda,y=features))+geom_point(size=2)+geom_line(data=scenarioCPCA,aes(x=lambda,y=Mlambda),linetype="dashed",size=2,colour="blue")+ylab("Number of selected variables")+xlab(expression(lambda))+ggtitle("PCA on Colon Cancer Dataset")
dev.off()

###################################
## Consider constrained version
##########################
#Find eigenvector and eigenvalue of Q
library(irlba)
tmp=irlba(Q,nu=1,nv=0)
l=tmp$u
gamma=tmp$d
tau_intervalCPCA=seq(1,sum(abs(l)),by=(sum(abs(l)))/50)
featuresCPCAtau=rep(0,length(tau_intervalCPCA))
source("Algorithm1.R")

#Solve using Algorithm 2 for the range of tau_values, keep track of the number of selected features
for (i in 1:length(tau_intervalCPCA)){
    cat(i)
    tau=tau_intervalCPCA[i]
    v=algorithm2(Q,tau)
    featuresCPCAtau[i]=sum(v!=0)
}

#Make a plot of number of selected variables versus tuning parameter for Scenario 2
library(ggplot2)
pdf(file="ColonCancerPCAconstrained.pdf",width=4,height=4)
scenarioCPCAtau=data.frame(lambda=-tau_intervalCPCA,features=featuresCPCAtau)
ggplot(scenarioCPCAtau,aes(x=lambda,y=features))+geom_point(size=2)+ylab("Number of selected variables")+xlab(expression(-tau))+ggtitle("PCA on Colon Cancer Dataset")
dev.off()

