#include <stdio.h>
#include <stdlib.h>
#include <math.h>

//function for maximum
double max(double a, double b){
    if (a>=b){
        return a;
    }else{
        return b;
    }
}


//function for inner product of two vectors
double mul(double *a, double *b, int* length){
    int index;
    double result=0;
    
    for (index = 0; index < *length; index++){
        result+=a[index] * b[index];
    }
    return result;
}

//calculate l1 norm of vector v
double computel1norm(double *v,int *p){
    int k;
    double norm=0;
    
    for (k=0;k<*p;k++){
        norm+=fabs(v[k]);
    }
    return(norm);
}


//Binsearch for tuning parameter for l1 constrained problem
// lambda - current lambda value
// tmp - vector to threshold, size p
// lmax - maximal tuning parameter
// l1norm - norm after initial threshold
// tau - parameter for the constraint
// p - number of variables
// eps - precision threshold
// max_iter - maximal number of iterations
// iter - current number of iterations
void binsearch(double *lambda, double *tmp, double *lmax,double *l1norm,double *tau, int *p, double *eps, int *max_iter,int *iter){

    int k;
    double lmin=0,normv;
    double v[*p]; //this is a vector of length p
    
    *iter=0;
    do{
      ++*iter;
        if (*l1norm>*tau){
            // adjust lmin
            lmin=*lambda;
        }else{
            // adjust lmax
            *lmax=*lambda;
        }
        // adjust lambda
        *lambda=(*lmax+lmin)/2;
        // calculate new v by soft-thresholding each element of tmp
        for (k=0;k<*p;k++){
            v[k]=((tmp[k]>0)-(tmp[k]<0))*max(0,fabs(tmp[k])-(*lambda)/2);
        }
        // standardize v if it's not zero
        normv=sqrt(mul(&v[0],&v[0],p));
        if (normv!=0){
            for (k=0;k<*p;k++){
                v[k]=v[k]/normv;
            }
        }
        // update l1norm
        *l1norm=computel1norm(&v[0],p);

    }while ((fabs(*l1norm-*tau)>*eps)&&(*iter<*max_iter));
}
